from autogluon.core.utils.early_stopping import ES_CLASS_MAP, AdaptiveES


# TODO: Add more strategies
def get_early_stopping_rounds(num_rows_train, strategy="auto", min_patience=20, max_patience=300, min_rows=10000):
    if isinstance(strategy, (tuple, list)):
        strategy = list(strategy)
        if isinstance(strategy[0], str):
            if strategy[0] in ES_CLASS_MAP:
                strategy[0] = ES_CLASS_MAP[strategy[0]]
            else:
                raise AssertionError(f"unknown early stopping strategy: {strategy}")
        return strategy

    """Gets early stopping rounds"""
    if strategy == "auto":
        strategy = "simple"

    modifier = 1 if num_rows_train <= min_rows else min_rows / num_rows_train
    simple_early_stopping_rounds = max(
        round(modifier * max_patience),
        min_patience,
    )
    if strategy == "simple":
        return simple_early_stopping_rounds
    elif strategy == "adaptive":
        return AdaptiveES, dict(adaptive_offset=min_patience, min_patience=simple_early_stopping_rounds)
    else:
        raise AssertionError(f"unknown early stopping strategy: {strategy}")



#!/usr/bin/env python
###########################
# This code block is a HACK (!), but is necessary to avoid code duplication. Do NOT alter these lines.
import importlib.util
import os

from setuptools import setup

filepath = os.path.abspath(os.path.dirname(__file__))
filepath_import = os.path.join(filepath, "..", "core", "src", "autogluon", "core", "_setup_utils.py")
spec = importlib.util.spec_from_file_location("ag_min_dependencies", filepath_import)
ag = importlib.util.module_from_spec(spec)
# Identical to `from autogluon.core import _setup_utils as ag`, but works without `autogluon.core` being installed.
spec.loader.exec_module(ag)
###########################

version = ag.load_version_file()
version = ag.update_version(version)

submodule = "core"
install_requires = (
    [
        # version ranges added in ag.get_dependency_version_ranges()
        "numpy",
        "scipy",
        "scikit-learn",
        "networkx",
        "pandas",
        "tqdm",
        "requests",
        "matplotlib",
        "boto3",
        f"autogluon.common=={version}",
    ]
    if not ag.LITE_MODE
    else [
        # version ranges added in ag.get_dependency_version_ranges()
        "numpy",
        "scipy",
        "scikit-learn",
        "pandas",
        "tqdm",
        "matplotlib",
        f"{ag.PACKAGE_NAME}.common=={version}",
    ]
)

extras_require = {
    "ray": [
        "ray[default]>=2.3,<2.4",
        # https://github.com/grpc/grpc/issues/31885
        # version range set to align with ray's updated version range:
        # https://github.com/ray-project/ray/blob/master/python/setup.py#L259-L261
        # TODO: remove the pin once issue resolved
        "grpcio>=1.42.0,<=1.50.0",
        "pydantic>=1.10.4,<2.0",  # https://github.com/ray-project/ray/issues/36990
    ],
    "raytune": [
        "ray[tune]>=2.3,<2.4",
        # TODO: consider alternatives as hyperopt is not actively maintained.
        "hyperopt>=0.2.7,<0.2.8",  # This is needed for the bayes search to work.
        # 'GPy>=1.10.0,<1.11.0'  # TODO: Enable this once PBT/PB2 are supported by ray lightning
    ],
}

tests_require = [
    "pytest",
    "types-requests",
    "types-setuptools",
    "pytest-mypy",
]

all_requires = []

for extra_package in ["ray", "raytune"]:
    all_requires += extras_require[extra_package]
tests_require = list(set(tests_require))
all_requires = list(set(all_requires))
extras_require["tests"] = tests_require
extras_require["all"] = all_requires

install_requires = ag.get_dependency_version_ranges(install_requires)

if __name__ == "__main__":
    ag.create_version_file(version=version, submodule=submodule)
    setup_args = ag.default_setup_args(version=version, submodule=submodule)
    setup(
        install_requires=install_requires,
        extras_require=extras_require,
        **setup_args,
    )
